local json = require("json")
local utils = require("utils")
local const = require("const")
local pl_pretty = require("pl.pretty")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")


local function write_user( user_list )
    user_list_raw = json.encode(user_list)
    ok, msg = utils.set_data(const.data_key.eweb.user, user_list_raw)
    if (ok) then
        utils.error_exit(200)
    end
    utils.error_exit(500, "set user data failed")
end


local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("UserManage")

-- read user data
local ok, msg = utils.get_data(const.data_key.eweb.user)
if (not (ok and msg)) then
    utils.error_exit(500, "get user list failed")
end

local user_list = json.decode(msg)

if ("GET" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, 'invalid parameter')
    end
    username = args["Username"]
    utils.check_username(username)

    user_info = user_list["UserList"][username]
    if (nil == user_info) then
        utils.error_exit(200, '{}')
        -- utils.error_exit(400, 'bad user name')
    end

    res = {}
    res["Username"] = user_info["Username"]
    res["PasswordHash"] = user_info["PasswordHash"]
    res["Permission"] = user_info["Permission"]
    res["LastLoginTime"] = user_info["LastLoginTime"]

    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)

elseif ("DELETE" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, 'invalid parameter')
    end

    username = args["Username"]
    utils.check_username(username)

    if ("admin" == username) then
        utils.error_exit(400, "admin user can't be deleted")
    end

    if (nil == user_list["UserList"][username]) then
        utils.error_exit(400, "the user is inexistent")
    end

    user_list["UserList"][username] = nil
    write_user(user_list)

elseif ("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    if (nil == args_json) then
        utils.error_exit(400, "invalid parameter")
    end
    
    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end
    
    username = args["Username"]
    action = args["Action"]
    passwd = args["Password"]
    permission = args["Permission"]

    utils.check_username(username)
    utils.check_passwd(passwd) 
    utils.check_permission_format(permission)
    if (not ("Create" == action or "Update" == action)) then
        utils.error_exit(400, "invalid action parameter")
    end

    -- Action": string["Create", "Update"], //Action，表示具体的操作；Create：创建新用户；Update：更新用户
    if ("Create" == action) then
        if (nil ~= user_list["UserList"][username]) then
            utils.error_exit(400, "the user is already existent")
        end
        user_info = {}
        user_info["Username"] = username
        salt = resty_string.to_hex(resty_random.bytes(16))
        user_info["PasswordSalt"] = salt

        hex = utils.hmac_sha256(salt, passwd)
        if (nil == hex) then
            utils.log_error("hmac failed")
            utils.error_exit(500)
        end

        user_info["Password"] = passwd
        user_info["PasswordHash"] = resty_string.to_hex(hex)
        user_info["Permission"] = permission
        user_info["LastLoginTime"] = "0"
        user_list["UserList"][username] = user_info
    elseif ("Update" == action) then
        user_list["UserList"][username]["Username"] = username
        user_list["UserList"][username]["Permission"] = permission
        user_list["UserList"][username]["Password"] = passwd

        passwd_hash = user_list["UserList"][username]["PasswordHash"]
        if (passwd_hash ~= passwd) then
            --[[keeping the password and salt if password is unchanged --]]
            salt = resty_string.to_hex(resty_random.bytes(16))
            user_list["UserList"][username]["PasswordSalt"] = salt
            user_list["UserList"][username]["PasswordHash"] = resty_string.to_hex(utils.hmac_sha256(salt, passwd))
        end
    else
        utils.error_exit(400, "invalid parameter")
    end
    
    write_user(user_list)
end

utils.error_exit(400, "unsupported request method")
------ EOF ------
